package cn.sh.stc.sict.theme.hpgp.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.db.DaoTemplate;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import cn.sh.stc.sict.cloud.common.core.constant.RedisCacheConstant;
import cn.sh.stc.sict.cloud.common.core.util.NumberUtil;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.theme.hpgp.dao.HpgpBusyIdlePredictionMapper;
import cn.sh.stc.sict.theme.hpgp.dao.HpgpDepartmentRankMapper;
import cn.sh.stc.sict.theme.hpgp.model.HpgpBusyIdlePrediction;
import cn.sh.stc.sict.theme.hpgp.model.HpgpDepartmentRank;
import cn.sh.stc.sict.theme.hpgp.service.HpgpBusyIdlePredictionService;
import cn.sh.stc.sict.theme.hpgp.vo.DeptPredictionVO;
import cn.sh.stc.sict.theme.hpgp.vo.DeptRankVO;
import cn.sh.stc.sict.theme.hphy.wd.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 黄浦全科导诊——忙闲预测(HpgpBusyIdle_ prediction)表服务实现类
 *
 * @author zhangly
 * @since 2022-01-14 09:56:43
 */
@Slf4j
@AllArgsConstructor
@Service("hpgpBusyIdlPredictionService")
public class HpgpBusyIdlePredictionServiceImpl extends ServiceImpl<HpgpBusyIdlePredictionMapper, HpgpBusyIdlePrediction> implements HpgpBusyIdlePredictionService {
    private final HpgpDepartmentRankMapper departmentRankMapper;
    private final StringRedisTemplate stringRedisTemplate;

    @Override
    public List<DeptPredictionVO> busyIdlePrediction(String deptName) {
        // 查询科室排名
        // 组装医院代码、科室代码
        // 查询医院忙闲
        List<DeptRankVO> rankList = departmentRankMapper.getRankByStandardDept(deptName, 3);
        if (CollectionUtil.isEmpty(rankList)) {
            return null;
        }
        DateTime startTime = DateUtil.date();
        DateTime endTime = DateUtil.offsetDay(DateUtil.date(), 7);
        List<DeptPredictionVO> result = new ArrayList<>();
        rankList.forEach(rank -> {
            List<DeptPredictionVO> rankResult = new ArrayList<>();
            String key = RedisCacheConstant.DEPT_NUM_SOURCE_RESULT + ":" + rank.getHospitalCode();
            if (StrUtil.isNotBlank(rank.getOneDeptCode())) {
                key = key + ":" + rank.getOneDeptCode();
            }
            if (StrUtil.isNotBlank(rank.getSubDeptCode())) {
                key = key + ":" + rank.getSubDeptCode();
            }
            if (stringRedisTemplate.hasKey(key)) {
                String str = stringRedisTemplate.opsForValue().get(key);
                JSONArray array = JSONUtil.parseArray(str);
                result.addAll(JSONUtil.toList(array, DeptPredictionVO.class));
            } else {
                DeptInfo deptInfo = new DeptInfo();
                deptInfo.setHosOrgCode(rank.getHospitalCode());
                deptInfo.setOneDeptCode(rank.getOneDeptCode());
                deptInfo.setDeptCode(rank.getSubDeptCode());
                List<OutPatInfo> outPatInfo = WanDaHttpUtil.getResourceOutPatInfo(deptInfo);
                if (CollUtil.isNotEmpty(outPatInfo)) {
                    Map<String, DeptPredictionVO> dateResultMap = new HashMap<>();
                    outPatInfo.forEach(pat -> {
                        NumSourceInfo numSourceInfo = new NumSourceInfo();
                        numSourceInfo.setHosOrgCode(rank.getHospitalCode());
                        numSourceInfo.setOneDeptCode(rank.getOneDeptCode());
                        numSourceInfo.setDeptCode(rank.getSubDeptCode());
                        numSourceInfo.setStartTime(DateUtil.format(startTime, DatePattern.NORM_DATE_FORMAT));
                        numSourceInfo.setEndTime(DateUtil.format(endTime, DatePattern.NORM_DATE_FORMAT));
                        numSourceInfo.setOrderType(WanDaConstant.ORDER_TYPE_CLINC);
                        numSourceInfo.setPatient_type(WanDaConstant.PATIENT_TYPE_NORMAL);
                        numSourceInfo.setResourceCode(pat.getResourceCode());
                        List<NumSourceInfo> orderNumInfo = WanDaHttpUtil.getOrderNumInfo(numSourceInfo);
                        if (CollUtil.isNotEmpty(orderNumInfo)) {
                            Map<String, List<NumSourceInfo>> dateMap = orderNumInfo.stream().collect(Collectors.groupingBy(NumSourceInfo::getScheduleDate));
                            dateMap.forEach((date, numList) -> {
                                DeptPredictionVO vo = dateResultMap.getOrDefault(date, new DeptPredictionVO());
                                vo.setDeptCode(rank.getDeptCode());
                                vo.setDeptName(rank.getDeptName());
                                vo.setHospitalName(rank.getHospitalName());
                                vo.setHospitalCode(rank.getHospitalCode());
                                vo.setPredictionDate(DateUtil.parseDate(date));
                                int reserveOrderNum = numList.stream().mapToInt(NumSourceInfo::getReserveOrderNum).sum();
                                vo.setReserveOrderNum(NumberUtil.isNullOrZero(vo.getReserveOrderNum()) ? reserveOrderNum : vo.getReserveOrderNum() + reserveOrderNum);
                                int remain = numList.stream().mapToInt(NumSourceInfo::getRemainNum).sum();
                                vo.setRemainNum(NumberUtil.isNullOrZero(vo.getRemainNum()) ? remain : vo.getRemainNum() + remain);
                                vo.setScoreByNum();
                                dateResultMap.put(date, vo);
                            });
                        }
                    });
                    dateResultMap.forEach((k, v) -> {
                        rankResult.add(v);
                    });
                }
                // 生成无号源信息
                if (CollUtil.isEmpty(rankResult)) {
                    for (Date i = DateUtil.offsetDay(startTime, 1); DateUtil.compare(i, endTime) < 1; i = DateUtil.offsetDay(i, 1)) {
                        DeptPredictionVO vo = new DeptPredictionVO();
                        vo.setDeptCode(rank.getDeptCode());
                        vo.setDeptName(rank.getDeptName());
                        vo.setHospitalName(rank.getHospitalName());
                        vo.setHospitalCode(rank.getHospitalCode());
                        vo.setPredictionDate(i);
                        vo.setScoreByNum();
                        rankResult.add(vo);
                    }
                }
                stringRedisTemplate.opsForValue().set(key, JSONUtil.toJsonStr(rankResult), 4, TimeUnit.HOURS);
                result.addAll(rankResult);
            }
        });
        return result;
    }
}
