package cn.sh.stc.sict.theme.hpgp.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.json.JSONUtil;
import cn.sh.stc.sict.theme.hphy.wd.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.sh.stc.sict.theme.hpgp.dao.HpgpDepartmentRankMapper;
import cn.sh.stc.sict.theme.hpgp.model.HpgpDepartmentRank;
import cn.sh.stc.sict.theme.hpgp.service.HpgpDepartmentRankService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 黄浦全科导诊——科室排名(HpgpDepartmentRank)表服务实现类
 *
 * @author zhangly
 * @since 2022-01-14 09:56:47
 */
@Slf4j
@Service("hpgpDepartmentRankService")
public class HpgpDepartmentRankServiceImpl extends ServiceImpl<HpgpDepartmentRankMapper, HpgpDepartmentRank> implements HpgpDepartmentRankService {

    @Override
    public void updateResource() {
        List<HpgpDepartmentRank> deptList = this.list();
        deptList.forEach(dept -> {
            if (dept.getOneDeptCode().equals(dept.getDeptCode())) {
                return;
            }
            this.asyncUpdateRank(dept);
        });
    }

    @Async
    void asyncUpdateRank(HpgpDepartmentRank dept) {
        DeptInfo deptInfo = new DeptInfo();
        deptInfo.setHosOrgCode(dept.getHospitalCode());
        deptInfo.setOneDeptCode(dept.getOneDeptCode());
        deptInfo.setDeptCode(dept.getDeptCode());
        List<OutPatInfo> resourceOutPatInfo = WanDaHttpUtil.getResourceOutPatInfo(deptInfo);
        List<DoctInfo> resourceDoctInfo = WanDaHttpUtil.getResourceDoctInfo(deptInfo);
        if (CollUtil.isEmpty(resourceDoctInfo) && CollUtil.isEmpty(resourceOutPatInfo)) {
            return;
        }
        HpgpDepartmentRank update = new HpgpDepartmentRank();
        update.setId(dept.getId());
        update.setResourceDoctInfo(JSONUtil.toJsonStr(resourceDoctInfo));
        update.setResourceOutPatInfo(JSONUtil.toJsonStr(resourceOutPatInfo));
        this.updateById(update);
    }

}
